/*
 * ROBOT - tank
 * (c) 2009, Adam Heinrich, <www.adamh.cz>
 * http://knowhow.adamh.cz/tank
 * 
 * ATmega48 @ 8MHz    
 */
#define F_CPU 8000000  // 1 MHz

#define BAUDRATE 9600

#define MYUBRR F_CPU/16/BAUDRATE-1

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include "uart.h"

#define LED_OFF() PORTD &= ~(1<<PD7);
#define LED_ON() PORTD |= (1<<PD7);

#define leftStop() PORTB &= ~(1<<PB1);PORTD &= ~(1<<PB2);
#define leftGo() PORTB |= (1<<PB1);PORTD &= ~(1<<PB2);
#define leftBack() PORTB |= (1<<PB2);PORTD &= ~(1<<PB1);

#define rightStop() PORTD &= ~(1<<PD3);PORTD &= ~(1<<PD4);
#define rightGo() PORTD |= (1<<PD3);PORTD &= ~(1<<PD4);
#define rightBack() PORTD |= (1<<PD4);PORTD &= ~(1<<PD3);

#define BUTTON1 ((PINB & (1 << PB6)) > 0 )
#define BUTTON2 ((PINB & (1 << PB7)) > 0 )
#define BUTTON3 ((PIND & (1 << PD5)) > 0 )
#define BUTTON4 ((PIND & (1 << PD6)) > 0 )

volatile int cmd = 0;
volatile unsigned char command1 = ' ';
volatile unsigned char command2 = ' ';

// format: 'A', 'B', '\n' (3 bytes)
ISR(USART_RX_vect) {
  cmd++;
  unsigned char rec = uartReceive();
  
  if (cmd == 1) {
    command1 = rec;
  } else if (cmd == 2) {
    command2 = rec;
  } else if (rec == '\n') {
    PORTD = 0;
    PORTB = 0;
    
    switch (command1) {
      case 'F': leftGo(); break;
      case 'S': leftStop(); break;
      case 'B': leftBack(); break;
    }
    
    switch (command2) {
      case 'F': rightGo(); break;
      case 'S': rightStop(); break;
      case 'B': rightBack(); break;
    }
    
    uartTransmit('O');
    uartTransmit('K');
	  uartTransmit('\n');
	  cmd = 0;
	}
}

int main(void) {
  // outputs setting
	DDRD |= (1<<PD7); // LED
	DDRB |= (1<<PB1); // motor 1.1
	DDRB |= (1<<PB2); // motor 1.2
	DDRD |= (1<<PD3); // motor 2.1
	DDRD |= (1<<PD4); // motor 2.2
	
	// prescaler to FOSC/1 (8 MHz)
	CLKPR  = 0b10000000; // set CLKPCE bit to 1
	CLKPR  = 0b00000000; // set all other bits to 0 => divide by 1
	
	// UART initialization, enable interrupts
  uartInit(MYUBRR);
	sei();
	
	// welcome message
	uartTransmit('R');
	uartTransmit('o');
	uartTransmit('b');
	uartTransmit('o');
	uartTransmit('t');
  uartTransmit('C');
	uartTransmit('o');
	uartTransmit('n');
	uartTransmit('t');
	uartTransmit('r');
	uartTransmit('o');
	uartTransmit('l');
	uartTransmit('\n');
	uartTransmit('m');
	uartTransmit('o');
	uartTransmit('t');
	uartTransmit('o');
	uartTransmit('r');
	uartTransmit('s');
	uartTransmit(':');
	uartTransmit(' ');
	uartTransmit('L');
	uartTransmit('R');
	uartTransmit('\n');
	uartTransmit('L');
	uartTransmit('/');
	uartTransmit('R');
	uartTransmit(':');
	uartTransmit(' ');
	uartTransmit('F');
	uartTransmit('|');
	uartTransmit('S');
	uartTransmit('|');
	uartTransmit('B');
	uartTransmit('\n');
	uartTransmit('\n');
	
	int i = 1;

  while(1) {
    if (i) {
      LED_ON();
      i = 0;
    } else {
      LED_OFF();
      i = 1;
    }

    _delay_ms(200);
  }
}