/**
 * USART initialization
 */
void uartInit(unsigned int ubrr) {
  	// set baud rate
  	UBRR0H = (unsigned char)(ubrr>>8);
  	UBRR0L = (unsigned char)ubrr;
	// Enable receiver, transmitter and reciever interrupt
	UCSR0B = (1<<RXEN0)|(1<<TXEN0)|(1<<RXCIE0);
	// set frame format: 8 data bits, 1 stop bit, no parity */
	UCSR0C = (0<<USBS0)|(3<<UCSZ00);
}

/**
 * Data receiving
 */
unsigned char uartReceive() {
	// wait for data to be received
	while ( !(UCSR0A & (1<<RXC0)) )
		;
  	// get and return received data from buffer */
  	return UDR0;
}

/**
 * Data transmiting
 */
void uartTransmit(unsigned char data) {
	/* Wait for empty transmit buffer */
	while ( !( UCSR0A & (1<<UDRE0)) )
		;
	/* Put data into buffer, sends the data */
	UDR0 = data;
}

/**
 * Enabling interrupt
 */
void enableUartInterrupt() {
	UCSR0B = (1<<RXEN0)|(1<<TXEN0)|(1<<RXCIE0);
}

/**
 * Disabling interrupt
 */
void disableUartInterrupt() {
	UCSR0B = (1<<RXEN0)|(1<<TXEN0);
}
